__version__ = "1.0.4"


class APP(object):
    name = "colorize"
    program = 'python -m colorize'
    version = __version__
    description = "Command line utility to colorize other commands output"



#!/usr/bin/env python

import sys
import argparse
import logging

from .colorize import Configuration
from .colorize import Colorize
from . import APP


def logging_setup(args):
    def set_format(logger, format, level=logging.INFO):
        formatter = logging.Formatter(format, args.datefmt)
        handler = logging.StreamHandler()
        handler.setFormatter(formatter)
        logger.addHandler(handler)
        logger.setLevel(level)
    logger = logging.getLogger('colorize.main')
    shlogger = logging.getLogger('colorize.shell')

    set_format(logger, '%(message)s')
    set_format(shlogger, args.format)


def main():
    parser = argparse.ArgumentParser(
        prog=APP.program,
        description=APP.description,
        )
    parser.add_argument('command', nargs='*',
                        help='Command to be executed')
    parser.add_argument('-f', '--format', default='%(message)s',
                        help='Configures both stdout and stderr')
    parser.add_argument('--date-format', dest="datefmt",
                        default='%m-%d %H:%M:%S',
                        help='Set the date format')

    args = parser.parse_args()

    logging_setup(args)

    conf = Configuration()
    conf.process()
    colorize = Colorize(conf)
    colorize.process(args.command)
    sys.exit(colorize.return_code)


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright (C) 2012-2014 Miguel Angel Garcia <miguelangel.garcia@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import re
import csv
import subprocess
import logging
from threading import Thread

from . import APP


logger = logging.getLogger('colorize.main')
shlogger = logging.getLogger('colorize.shell')


class Color(object):
    COLOR = '\033[{}m'
    NORMAL = COLOR.format('')
    colors = {
        'black':       '0',
        'red':         '1',
        'green':       '2',
        'brown':       '3',
        'blue':        '4',
        'magenta':     '5',
        'cyan':        '6',
        'white':       '7',
        }

    def __init__(self, bold=False, foreground=None, background=None):
        self.bold = bold
        self.foreground = foreground
        self.background = background

    def __str__(self):
        properties = []

        properties.append(self._getBoldString())
        if self.foreground:
            properties.append(self._getForegroundString())
        if self.background:
            properties.append(self._getBackgroundString())

        return self.COLOR.format(';'.join(properties))

    def _getBoldString(self):
        if self.bold:
            return '1'
        return '0'

    def _getForegroundString(self):
        return '3' + self.colors[self.foreground]

    def _getBackgroundString(self):
        return '4' + self.colors[self.background]


class Configuration(object):
    def __init__(self):
        self.regexp = {}

    def process(self):
        for filename in [self.configfile_currentdir(),
                         self.configfile_home(),
                         self.configfile_default()]:
            if os.path.exists(filename):
                self._parse_config(filename)
                break

    def configfile_currentdir(self):
        return '.{}.conf'.format(APP.name)

    def configfile_home(self):
        return os.path.expanduser(
            '~/.config/{0}/{0}.conf'.format(APP.name))

    def configfile_default(self):
        return '/etc/{0}/{0}.conf'.format(APP.name)

    def _parse_config(self, filename):
        with open(filename) as fd:
            reader = csv.reader(fd)
            for row in reader:
                if not row or (row[0] and row[0][0] == '#'):
                    continue

                row += 4*[None]

                regexp = row[0]
                color = Color()
                if row[1]:
                    color.bold = row[1].strip().lower() in ['1', 'true']
                if row[2]:
                    color.foreground = row[2].strip()
                if row[3]:
                    color.background = row[3].strip()

                self.regexp[regexp] = color


class Printer(object):
    def __init__(self, fdin, regexps, log_fn):
        self.fdin = fdin
        self.regexps = regexps
        self.log_fn = log_fn

    def process(self):
        while not self.fdin.closed:
            line = self.fdin.readline()
            if isinstance(line, bytes):
                line = line.decode('utf-8')
            if line == '':
                break
            self.log_fn(self._replace(line.rstrip()))

    def _replace(self, line):
        result = line
        for exp, color in self.regexps.items():
            result, _ = re.subn(exp, color, result)
        return result


class PrinterThread(Thread):
    def __init__(self, fn):
        super(PrinterThread, self).__init__()
        self._fn = fn
        self.start()

    def run(self):
        self._fn()

    def flush(self):
        Thread.join(self, 1)


class Colorize(object):
    def __init__(self, config):
        self.regexp = config.regexp
        self.regexps = {}
        self.return_code = 0

    def process(self, command):
        if command:
            return self.process_command(command)
        self.process_stdin()

    def process_command(self, command):
        self.compile_regexps()

        process = subprocess.Popen(command,
                                   stdin=subprocess.PIPE,
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE)
        outprinter = Printer(process.stdout, self.regexps, shlogger.info)
        errprinter = Printer(process.stderr, self.regexps, shlogger.error)
        outpid = PrinterThread(outprinter.process)
        errpid = PrinterThread(errprinter.process)
        process.wait()
        outpid.flush()
        errpid.flush()
        self.return_code = process.returncode

    def process_stdin(self):
        self.compile_regexps()

        printer = Printer(sys.stdin, self.regexps, shlogger.info)
        try:
            printer.process()
        except KeyboardInterrupt:
            logger.error('Interrupted by user')

    def compile_regexps(self):
        for exp, color in self.regexp.items():
            regexp = '({})'.format(exp)
            compiled = re.compile(regexp)
            self.regexps[compiled] = '{}\\1{}'.format(color, Color.NORMAL)



import unittest
import doublex
from colorize import colorize
from tests.utils import FakeIO


class PrinterTests(unittest.TestCase):
    def test_reads_a_line_and_prints_it(self):
        line = 'foo'
        stdin = FakeIO(line)
        with doublex.Mock() as log:
            log.log(line)

        sut = colorize.Printer(stdin, {}, log.log)

        sut.process()

        doublex.assert_that(log, doublex.verify())

    def test_reads_two_lines_and_prints_them(self):
        line = '1\n2'
        stdin = FakeIO(line)
        with doublex.Mock() as log:
            log.log('1')
            log.log('2')

        sut = colorize.Printer(stdin, {}, log.log)

        sut.process()

        doublex.assert_that(log, doublex.verify())

    def test_replacement(self):
        line = 'foo'
        stdin = FakeIO(line)
        expected = 'ok'
        with doublex.Mock() as log:
            log.log(expected)
        regexps = {'foo': expected}

        sut = colorize.Printer(stdin, regexps, log.log)

        sut.process()

        doublex.assert_that(log, doublex.verify())



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
from colorize import APP


def read_description():
    with open('README.rst') as fd:
        return fd.read()


setup(name='colorize',
      version=APP.version,
      description=APP.description,
      long_description=read_description(),
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.2',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: MIT License',
          'Operating System :: OS Independent',
          'Topic :: Software Development :: User Interfaces',
      ],
      keywords='interface,command line',
      author='Miguel Ángel García',
      author_email='miguelangel.garcia@gmail.com',
      url='https://github.com/magmax/colorize',
      license='MIT',
      packages=find_packages(exclude=['tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
      ],
      entry_points={
          'setuptools.installation': [
              "eggsecutable = colorize.bar:foo"
          ]
      },
      data_files=[
          ('', ['__main__.py', ]),
      ],
  )



#!/usr/bin/python
# -*- coding:utf-8; tab-width:4; mode:python -*-

import unittest
from colorize import colorize


class ColorTests(unittest.TestCase):
    def test_getNormal(self):
        expected = '\033[m'

        self.assertEquals(colorize.Color.NORMAL, expected)

    def test_getBlack(self):
        sut = colorize.Color(foreground='black')
        expected = '\033[0;30m'
        current = str(sut)

        self.assertEquals(current, expected)

    def test_getWhite(self):
        sut = colorize.Color(foreground='white')
        expected = '\033[0;37m'
        current = str(sut)

        self.assertEquals(current, expected)

    def test_getBoldWhite(self):
        sut = colorize.Color(bold=True, foreground='white')
        expected = '\033[1;37m'
        current = str(sut)

        self.assertEquals(current, expected)

    def test_getBlue(self):
        sut = colorize.Color(foreground='blue')
        expected = '\033[0;34m'
        current = str(sut)

        self.assertEquals(current, expected)

    def test_getGreen(self):
        sut = colorize.Color(foreground='green')
        expected = '\033[0;32m'
        current = str(sut)

        self.assertEquals(current, expected)

    def test_getCyan(self):
        sut = colorize.Color(foreground='cyan')
        expected = '\033[0;36m'
        current = str(sut)

        self.assertEquals(current, expected)

    def test_getRed(self):
        sut = colorize.Color(foreground='red')
        expected = '\033[0;31m'
        current = str(sut)

        self.assertEquals(current, expected)

    def test_getMagenta(self):
        sut = colorize.Color(foreground='magenta')
        expected = '\033[0;35m'
        current = str(sut)

        self.assertEquals(current, expected)

    def test_getBrown(self):
        sut = colorize.Color(foreground='brown')
        expected = '\033[0;33m'
        current = str(sut)

        self.assertEquals(current, expected)

    def test_getBoldBlue(self):
        sut = colorize.Color(foreground='blue', bold=True)
        expected = '\033[1;34m'
        current = str(sut)

        self.assertEquals(current, expected)

    def test_getBoldGreen(self):
        sut = colorize.Color(foreground='green', bold=True)
        expected = '\033[1;32m'
        current = str(sut)

        self.assertEquals(current, expected)

    def test_getBoldCyan(self):
        sut = colorize.Color(foreground='cyan', bold=True)
        expected = '\033[1;36m'
        current = str(sut)

        self.assertEquals(current, expected)

    def test_getBoldRed(self):
        sut = colorize.Color(foreground='red', bold=True)
        expected = '\033[1;31m'
        current = str(sut)

        self.assertEquals(current, expected)

    def test_getBoldMagenta(self):
        sut = colorize.Color(foreground='magenta', bold=True)
        expected = '\033[1;35m'
        current = str(sut)

        self.assertEquals(current, expected)

    def test_getBoldBrown(self):
        sut = colorize.Color(foreground='brown', bold=True)
        expected = '\033[1;33m'
        current = str(sut)

        self.assertEquals(current, expected)

    def test_getBrownBackground(self):
        sut = colorize.Color(background='brown')
        expected = '\033[0;43m'
        current = str(sut)

        self.assertEquals(current, expected)


if __name__ == '__main__':
    for x in ['black', 'red', 'green', 'brown',
              'blue', 'magenta', 'cyan', 'white']:
        color = colorize.Color(foreground=x)
        print(color, x, colorize.Color.NORMAL)

    print('--')
    for x in ['black', 'red', 'green', 'brown',
              'blue', 'magenta', 'cyan', 'white']:
        color = colorize.Color(bold=True, foreground=x)
        print(color, x, colorize.Color.NORMAL)

    print('--')
    for x in ['black', 'red', 'green', 'brown',
              'blue', 'magenta', 'cyan', 'white']:
        color = colorize.Color(background=x)
        print(color, x, colorize.Color.NORMAL)
